LIBRARY ieee;
USE ieee.std_logic_1164.ALL;
use IEEE.std_logic_textio.all;
use STD.textio.all;
use IEEE.numeric_std.all;
use ieee.math_real.all;

entity FT2232H_minimal_bench is
end FT2232H_minimal_bench;

architecture Behavioral of FT2232H_minimal_bench is

component FT2232H_minimal_A is Port(
	clk100: in std_logic;
	-- FT2232H
	D: out std_logic_vector(7 downto 0);
	RXF: in std_logic;
	TXE: in std_logic;
	RD: out std_logic;
	WR: out std_logic;
	SIWU: out std_logic;
	CLKOUT: in std_logic; --60MHz FTDI
	OE: out std_logic;
	PWRSAV: out std_logic);
end component;

component FT2232H_minimal_B is Port(
	clk100: in std_logic;
	-- FT2232H
	D: out std_logic_vector(7 downto 0);
	RXF: in std_logic;
	TXE: in std_logic;
	RD: out std_logic;
	WR: out std_logic;
	SIWU: out std_logic;
	CLKOUT: in std_logic; --60MHz FTDI
	OE: out std_logic;
	PWRSAV: out std_logic);
end component;

component FT2232H_minimal_C is Port(
	clk100: in std_logic;
	-- FT2232H
	D: out std_logic_vector(7 downto 0);
	RXF: in std_logic;
	TXE: in std_logic;
	RD: out std_logic;
	WR: out std_logic;
	SIWU: out std_logic;
	CLKOUT: in std_logic; --60MHz FTDI
	OE: out std_logic;
	PWRSAV: out std_logic);
end component;

signal clk100: std_logic:='0';
signal CLKOUT: std_logic:='0';
signal TXE: std_logic:='0';
signal A_D: std_logic_vector(7 downto 0):=(others => '0');
signal A_WR : std_logic:='0';
signal A_FT2232H_RCV: std_logic:='0';
signal A_D_buff: std_logic_vector(7 downto 0):=(others => '0');
signal A_fail: std_logic:='0';
signal B_D: std_logic_vector(7 downto 0):=(others => '0');
signal B_WR : std_logic:='0';
signal B_FT2232H_RCV: std_logic:='0';
signal B_D_buff: std_logic_vector(7 downto 0):=(others => '0');
signal B_fail: std_logic:='0';
signal C_D: std_logic_vector(7 downto 0):=(others => '0');
signal C_WR : std_logic:='0';
signal C_FT2232H_RCV: std_logic:='0';
signal C_D_buff: std_logic_vector(7 downto 0):=(others => '0');
signal C_fail: std_logic:='0';
signal FT2232H_not_full: std_logic:='0';
signal rand_num_1k : integer := 500;

begin

uut_A: FT2232H_minimal_A port map(
	clk100 => clk100,
	-- FT2232H
	D => A_D,
	RXF => '1',
	TXE => TXE,
	RD => open,
	WR => A_WR,
	SIWU => open,
	CLKOUT => CLKOUT,
	OE => open,
	PWRSAV => open);
	
uut_B: FT2232H_minimal_B port map(
	clk100 => clk100,
	-- FT2232H
	D => B_D,
	RXF => '1',
	TXE => TXE,
	RD => open,
	WR => B_WR,
	SIWU => open,
	CLKOUT => CLKOUT,
	OE => open,
	PWRSAV => open);
	
uut_C: FT2232H_minimal_C port map(
	clk100 => clk100,
	-- FT2232H
	D => C_D,
	RXF => '1',
	TXE => TXE,
	RD => open,
	WR => C_WR,
	SIWU => open,
	CLKOUT => CLKOUT,
	OE => open,
	PWRSAV => open);

clk100 <= not clk100 after 5ns;
CLKOUT <= not CLKOUT after 8.333ns;
FT2232H_not_full <= not FT2232H_not_full after rand_num_1k*1ns;

process
    variable seed1, seed2: positive; -- seed values for random generator
    variable rand: real; -- random real-number value in range 0 to 1.0  
	begin
    uniform(seed1, seed2, rand); -- generate random number
    rand_num_1k <= integer(rand*1000.0); -- rescale to 0..1000, convert integer part
    wait for 100 ns;
end process;

process
begin
	wait until rising_edge(CLKOUT);
	TXE <= '1';
	if FT2232H_not_full = '1' then
		TXE <= '0';
	end if;
end process;

process begin	
	wait until rising_edge(CLKOUT);
	A_fail <= '0';
	B_fail <= '0';
	C_fail <= '0';
	A_FT2232H_RCV <= '0';
	B_FT2232H_RCV <= '0';
	C_FT2232H_RCV <= '0';
	if A_WR = '0' and TXE = '0' then
		A_FT2232H_RCV <= '1';
		A_D_buff <= A_D;
		if unsigned(A_D) /= unsigned(A_D_buff)+1 then
			A_fail <= '1';
		end if;
	end if;
	if B_WR = '0' and TXE = '0' then
		B_FT2232H_RCV <= '1';
		B_D_buff <= B_D;
		if unsigned(B_D) /= unsigned(B_D_buff)+1 then
			B_fail <= '1';
		end if;
	end if;
	if C_WR = '0' and TXE = '0' then
		C_FT2232H_RCV <= '1';
		C_D_buff <= C_D;
		if unsigned(C_D) /= unsigned(C_D_buff)+1 then
			C_fail <= '1';
		end if;
	end if;
end process;					   

end Behavioral;